<?php
/*
Plugin Name: Google Voicemails to Custom Post Types
Plugin URI: http://ottopress.com/wordpress-plugins/google-voice/
Description: Reads your Google Voice account and turns your voicemails into a voicemail Custom Post type
Author: Otto
Version: 1.0
Author URI: http://ottodestruct.com
License: GPL2

    Copyright 2011  Samuel Wood  (email : otto@ottodestruct.com)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License version 2,
    as published by the Free Software Foundation.

    You may NOT assume that you can use any other version of the GPL.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    The license for this software can likely be found here:
    http://www.gnu.org/licenses/gpl-2.0.html

*/

define('GV_CHECK_TIME',5*60); // how often to check in seconds

include (ABSPATH . 'wp-admin/includes/admin.php');

add_action( 'init', 'gv_register_cpt_voicemail' );
function gv_register_cpt_voicemail() {

    $labels = array( 
        'name' => _x( 'Voicemails', 'gv' ),
        'singular_name' => _x( 'Voicemail', 'gv' ),
        'add_new' => _x( 'Add New', 'gv' ),
        'add_new_item' => _x( 'Add New Voicemail', 'gv' ),
        'edit_item' => _x( 'Edit Voicemail', 'gv' ),
        'new_item' => _x( 'New Voicemail', 'gv' ),
        'view_item' => _x( 'View Voicemail', 'gv' ),
        'search_items' => _x( 'Search Voicemails', 'gv' ),
        'not_found' => _x( 'No voicemails found', 'gv' ),
        'not_found_in_trash' => _x( 'No voicemails found in Trash', 'gv' ),
        'parent_item_colon' => _x( 'Parent Voicemail:', 'gv' ),
        'menu_name' => _x( 'Voicemails', 'gv' ),
    );

    $args = array( 
        'labels' => $labels,
        'hierarchical' => false,
        'description' => 'Incoming Voicemails',
        'supports' => array( 'title', 'editor', 'author', 'comments' ),
        'taxonomies' => array( 'category', 'post_tag' ),
        'public' => true,
        'show_ui' => true,
        'show_in_menu' => true,
        'show_in_nav_menus' => true,
        'publicly_queryable' => true,
        'exclude_from_search' => true,
        'has_archive' => true,
        'query_var' => true,
        'can_export' => true,
        'rewrite' => true,
        'capability_type' => 'post'
    );

    register_post_type( 'voicemail', $args );
}

add_action('init','gv_sync_voicemails'); // the voicemail sync has its own timer for checking and such.. cron is complicated

// add the admin options page
add_action('admin_menu', 'gv_admin_add_page');

function gv_admin_add_page() {
	add_options_page('Google Voice test', 'Google Voicemail Setup', 'manage_options', 'gv', 'gv_options_page');
}

function gv_options_page() {
?>
	<div class="wrap">
	<h2><?php _e('Google Voicemail Setup', 'gv'); ?></h2>
	<p><?php _e('Options relating to the Google Voicemail plugin.', 'gv'); ?> </p>
	<form method="post" action="options.php">
	<?php settings_fields('gv_options'); ?>
	<table><tr><td>
	<?php do_settings_sections('gv'); ?>
	</td><td style='vertical-align:top;'>
	<div style='width:20em; float:right; background: #ffc; border: 1px solid #333; margin: 2px; padding: 5px'>
			<h3 align='center'><?php _e('About the Author', 'sfc'); ?></h3>
		<p><a href="http://ottopress.com/blog/wordpress-plugins/google-voice/">Google Voicemails to CPTs</a> is developed and maintained by <a href="http://ottodestruct.com">Otto</a>.</p>
			<p>He blogs at <a href="http://ottodestruct.com">Nothing To See Here</a> and <a href="http://ottopress.com">Otto on WordPress</a>, posts photos on <a href="http://www.flickr.com/photos/otto42/">Flickr</a>, and chats on <a href="http://twitter.com/otto42">Twitter</a>.</p>
			<p>You can follow his site on either <a href="http://www.facebook.com/apps/application.php?id=116002660893">Facebook</a> or <a href="http://twitter.com/ottodestruct">Twitter</a>, if you like.</p>
			<p>If you'd like to <a href="https://www.paypal.com/cgi-bin/webscr?cmd=_donations&business=otto%40ottodestruct%2ecom">buy him a beer</a>, then he'd be perfectly happy to drink it.</p>
		</div>
	</td></tr></table>
	<?php submit_button(); ?>
	</form>
	</div>
	
<?php
}

add_action('admin_init', 'gv_admin_init'); 
function gv_admin_init() {
	$options = get_option('gv_options');
	register_setting( 'gv_options', 'gv_options', 'gv_options_validate' );
	add_settings_section('gv_main', __('Main Settings', 'gv'), 'gv_section_text', 'gv');
	add_settings_field('gv_email', __('Email', 'gv'), 'gv_email', 'gv', 'gv_main');
	add_settings_field('gv_pass', __('Password', 'gv'), 'gv_pass', 'gv', 'gv_main');
}


function gv_section_text() {
?>
<p><?php _e('In order to connect to Google Voice, the plugin needs your username and password.','gv'); ?></p>
<p><?php _e('If you use Google 2-step verification, please create an <a href="https://www.google.com/accounts/b/0/IssuedAuthSubTokens">Application specific password</a> for the plugin to use.','gv'); ?></p>
<?php
}

function gv_email() {
	$options = get_option('gv_options');
	?><input type="email" size="100" name='gv_options[email]' value="<?php echo $options['email']; ?>" /><?php
}

function gv_pass() {
	$options = get_option('gv_options');
	?><input type="password" size="100" name='gv_options[pass]' value="<?php echo $options['pass']; ?>" /><?php
}

function gv_options_validate($input) {
	$options = get_option('gv_options');

	if (is_email($input['email'])) { 
		$options['email'] = $input['email'];
	}
	
	if (isset($input['pass'])) { 
		$options['pass'] = $input['pass'];
	}
	
	return $options;
}

function gv_sync_voicemails() {
	$options = get_option('gv_options');
	if ($options['last_update_time'] && $options['last_update_time'] < time() - GV_CHECK_TIME ) return;

	if (empty($options['token'])) {
		$options['token'] = gv_clientlogin($options['email'],$options['pass']);
	}
	
	//get the gv info
	$msgs = gv_get_voicemails($options['token']);
	
	foreach($msgs as $msg) {
		$post=null;
		if ($msg['startTime']/1000 > $options['last_update_time']) {
			$post['post_type']='voicemail';
			$post['post_content']=$msg['messageText']."\n\n".
				"<audio src='https://www.google.com/voice/media/send_voicemail/{$msg['id']}' controls='controls'></audio>";
			$post['post_title']=$msg['displayStartDateTime'];
			$post['post_date']=date('Y-m-d H:i:s', $msg['startTime']/1000);
			$post['post_date_gmt']=gmdate('Y-m-d H:i:s', $msg['startTime']/1000);
			$post['post_status']='publish';
			
			$postid = wp_insert_post($post);
			
			if ($postid) {
				$post['ID'] = $postid;
				$url = gv_voicemail_sideload($msg['id'], $postid, $msg['messageText'], $options['token']);
				$post['post_content'] = $msg['messageText']."\n\n".
					"<audio src='{$url}' controls='controls'></audio>";
				wp_update_post($post);
			}
			
		}
	}
	
	$options['last_update_time'] = time();
	update_option('gv_options',$options);
}

function gv_clientlogin($email, $pass) {

	$options = array(
		'accountType'=>'HOSTED_OR_GOOGLE',
		'Email'=>$email,
		'Passwd'=>$pass,
		'service'=>'grandcentral',
		'source'=>'gv-wordpress-plugin',
	);
	$post = array('body' => $options, 'sslverify'=>false);

	$resp = wp_remote_post('https://google.com/accounts/ClientLogin', $post);
	$body = wp_remote_retrieve_body($resp);
	
	//look for the auth token
	if (preg_match('/Auth=([^\n]*)/', $body, $matches)) {
		return $matches[1];
	}
	
	return false;
}


function gv_get_voicemails($token) {
	$options['sslverify']=false;
	$options['headers']['Authorization'] = "GoogleLogin auth={$token}";
	$resp = wp_remote_get("https://www.google.com/voice/inbox/recent/voicemail/", $options);
	$body = wp_remote_retrieve_body($resp);
	$res = simplexml_load_string($body);
	$msgs = json_decode($res->json, true);
	$msgs = $msgs['messages'];
	return $msgs;
}

function gv_voicemail_sideload($gvid, $postid, $desc, $token) {
	$url = "https://www.google.com/voice/media/send_voicemail/{$gvid}";
	
	$tmpfname = wp_tempnam($url);
	if ( ! $tmpfname ) return false;

	$options['sslverify']=false;
	$options['headers']['Authorization'] = "GoogleLogin auth={$token}";
	$options['timeout'] = $timeout;
	$options['stream'] = true;
	$options['filename'] = $tmpfname;
	$response = wp_remote_get($url, $options);

	if ( is_wp_error( $response ) ) {
		unlink( $tmpfname );
		return false;
	}

	if ( 200 != wp_remote_retrieve_response_code( $response ) ){
		unlink( $tmpfname );
		return false;
	}

	$file_array['name'] = "{$gvid}.mp3";
	$file_array['tmp_name'] = $tmpfname;

	// do the validation and storage stuff
	$id = media_handle_sideload( $file_array, $postid, $desc );

	// If error storing permanently, unlink
	if ( is_wp_error($id) ) {
		@unlink($file_array['tmp_name']);
		return false;
	}

	$src = wp_get_attachment_url( $id );

	// Finally check to make sure the file has been saved, then return the html
	if ( ! empty($src) ) {
		$alt = isset($desc) ? esc_attr($desc) : '';
		return $src;
	}
	
	return false;
}

